<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Controller;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use A2G\A2gProducts\Services\FeuserSessionService;
use A2G\A2gProducts\Domain\Model\Ingredients;
use A2G\A2gProducts\Domain\Traits\{
    InjectIngredientsRepositoryTrait,
    InjectImageServiceTrait
};
use A2G\A2gProducts\PageTitle\A2gPageTitleProvider;
use TYPO3\CMS\Core\MetaTag\MetaTagManagerRegistry;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;

/**
 * This file is part of the "Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * IngredientsController
 */
class IngredientsController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

    use InjectIngredientsRepositoryTrait;
    use InjectImageServiceTrait;

    /**
     * action list
     *
     * @return string|object|null|void
     */
    public function listAction() {
        $ingredients = $this->ingredientsRepository->findAll();
        $this->view->assign('contentObjectUid', $this->configurationManager->getContentObject()->data['uid']);

        $this->view->assign('ingredients', $ingredients);
    }

    /**
     * action ajaxListAction
     *
     * @return string|object|null|void
     */
    public function ajaxListAction() {
        
    }

    /**
     * action show
     *
     * @param Ingredients $ingredient
     * @return string|object|null|void
     */
    public function showAction(Ingredients $ingredient) {
        $titleProvider = GeneralUtility::makeInstance(A2gPageTitleProvider::class);
        
        if(array_key_exists('pageTitleSuffix', $this->settings) && $this->settings['pageTitleSuffix'] !== null && $this->settings['pageTitleSuffix'] !== '' && (strlen($ingredient->getTitle()) + strlen($this->settings['pageTitleSuffix'])) < 66 ){
            $pageTitle = $ingredient->getTitle(). ' ' . $this->settings['pageTitleSuffix'];
        } else {
            $pageTitle = $ingredient->getTitle();
        }
        $titleProvider->setTitle($pageTitle);
        $contentObject = GeneralUtility::makeInstance(ContentObjectRenderer::class);
        $description = $contentObject->crop($ingredient->getDescription(), 125 . '|...|' . true);

        $uri = $this->uriBuilder->reset()->setCreateAbsoluteUri(true)->setTargetPageUid(
                        $GLOBALS['TSFE']->id)->uriFor('show', ['ingredient' => $ingredient], 'Ingredients', 'a2gProducts', 'ingredientsdetail');


        $this->view->assign('contentObjectUid', $this->configurationManager->getContentObject()->data['uid']);

        $metaTagManager = GeneralUtility::makeInstance(MetaTagManagerRegistry::class);

        /* TODO change
        $metaTagManager->getManagerForProperty('robots')->addProperty('robots', 'index, folow');
        $metaTagManager->getManagerForProperty('og:type')->addProperty('og:type', 'website');
        $metaTagManager->getManagerForProperty('twitter:card')->addProperty('twitter:card', 'summary_large_image');

        if ($ingredient->getTitle() !== '') {
            $metaTagManager->getManagerForProperty('title')->addProperty('title', $pageTitle);
            $metaTagManager->getManagerForProperty('og:title')->addProperty('og:title', $pageTitle);
            $metaTagManager->getManagerForProperty('twitter:title')->addProperty('twitter:title', $pageTitle);
        }
        if ($ingredient->getDescription() !== '') {
            $metaTagManager->getManagerForProperty('description')->addProperty('description', $description);
            $metaTagManager->getManagerForProperty('og:description')->addProperty('og:description', $description);
            $metaTagManager->getManagerForProperty('twitter:description')->addProperty('twitter:description', $description);
        }
        $metaTagManager->getManagerForProperty('og:url')->addProperty('og:url', $uri);
        $metaTagManager->getManagerForProperty('twitter:url')->addProperty('twitter:url', $uri);

        if ($ingredient->getImage() !== null) {
            $processedImage = $this->imageService->applyProcessingInstructions(
                    $this->imageService->getImage('', $ingredient->getImage(), false),
                    ['width' => '
        1200', 'height' => '630c']
            );
            $metaTagManager->getManagerForProperty('twitter:image')->addProperty('twitter:image', $this->imageService->getImageUri($processedImage, true));
            $metaTagManager->getManagerForProperty('og:image')->addProperty('og:image', $this->imageService->getImageUri($processedImage, true));
        }
        */
        $this->view->assign('ingredient', $ingredient);
    }

}
