<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Controller;

use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use A2G\A2gProducts\Services\FeuserSessionService;
use A2G\A2gProducts\Domain\Traits\{
    InjectCategoriesRepositoryTrait,
    InjectProductsRepositoryTrait,
    InjectFiltersRepositoryTrait,
    InjectImageServiceTrait
};
use A2G\A2gProducts\PageTitle\A2gPageTitleProvider;
use TYPO3\CMS\Core\MetaTag\MetaTagManagerRegistry;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
use A2G\A2gToolkit\Domain\Traits\SetMetaTagsTrait;

/**
 * This file is part of the "Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * ProductsController
 */
class ProductsController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

    use InjectProductsRepositoryTrait;
    use InjectFiltersRepositoryTrait;
    use InjectCategoriesRepositoryTrait;
    use InjectImageServiceTrait;

    use SetMetaTagsTrait;
    /**
     * @var FeuserSessionService
     */
    protected $feuserSessionService = null;

    /**
     * @param FeuserSessionService $feuserSessionService
     */
    public function injectFeuserSessionService(FeuserSessionService $feuserSessionService) {
        $this->feuserSessionService = $feuserSessionService;
    }

    /**
     * mapInt
     * 
     * @param string $int
     * @return int|null
     */
    static function mapInt(string $int): ?int {
        if (MathUtility::canBeInterpretedAsInteger($int)) {
            return (int) $int;
        }
        return null;
    }

    /**
     * 
     * @param \TYPO3\CMS\Extbase\DomainObject\AbstractEntity $obj
     * @return int|null
     */
    static function mapGetUid(\TYPO3\CMS\Extbase\DomainObject\AbstractEntity $obj): ?int {
        return $obj->getUid();
    }

    /**
     * action list
     *
     * @return string|object|null|void
     */
    public function listAction() {
        $uniqid = $GLOBALS['TSFE']->id . '_' . $this->configurationManager->getContentObject()->data['uid'];
        $this->injectFeuserSessionService(new FeuserSessionService('a2gProducts_' . $uniqid . '_'));
        $activFilter = $this->feuserSessionService->get('filter');
        $searchField = $this->feuserSessionService->get('searchField');
        $catUids = array_filter(array_map('self::mapInt', explode(',', $this->settings['showCategories'])));

        $categories = $this->categoriesRepository->findByUids($catUids);
        $filters = $this->filtersRepository->getFromCategories($catUids);
        if (!empty($activFilter)) {
            if ((int) $this->settings['ajaxListLimit'] === 0) {
                $products = $this->productsRepository->filter($catUids, $activFilter, $searchField, 0, 0, $filters);
            } else {
                $products = $this->productsRepository->filter($catUids, $activFilter, $searchField, $this->settings['ajaxListLimit'] + 2, 0, $filters);
            }
        } else {
            if ((int) $this->settings['ajaxListLimit'] === 0) {
                $products = $this->productsRepository->getFromCategories($catUids);
            } else {
                $products = $this->productsRepository->getFromCategories($catUids, (int) $this->settings['ajaxListLimit'] + 2);
            }
        }
            $this->view->assign('contentObjectUid', $this->configurationManager->getContentObject()->data['uid']);

        $this->view->assign('categories', $categories);
        $this->view->assign('filters', $filters);
        $this->view->assign('activeFilter', $activFilter);
        $this->view->assign('searchField', $searchField);
        $this->view->assign('products', $products);
        $this->view->assign('uniqid', $uniqid);
    }

    /**
     * action ajaxListAction
     *
     * @return string|object|null|void
     */
    public function ajaxListAction() {
        $this->injectFeuserSessionService(new FeuserSessionService('a2gProducts_' . (string) $_POST['tx_a2gproducts_a2gproductlist']['uniqid'] . '_'));
        $uniqid = $GLOBALS['TSFE']->id . '_' . $this->configurationManager->getContentObject()->data['uid'];
        $settings = [
            'detailPage' => $_POST['tx_a2gproducts_a2gproductlist']['dp'],
            'showCategories' => $_POST['tx_a2gproducts_a2gproductlist']['sc'],
            'style' => $_POST['tx_a2gproducts_a2gproductlist']['style'],
            'ajaxListLimit' => (int) $_POST['tx_a2gproducts_a2gproductlist']['limit']
        ];

        $catUids = array_filter(array_map('self::mapInt', explode(',', $_POST['tx_a2gproducts_a2gproductlist']['sc'])));
        $filters = $this->filtersRepository->getFromCategories($catUids);

	if (array_key_exists('filter',$_POST['tx_a2gproducts_a2gproductlist']) && $_POST['tx_a2gproducts_a2gproductlist']['filter'] === null) {
            $filter = [];
        } else {
            $filter = $_POST['tx_a2gproducts_a2gproductlist']['filter'];
        }

        $this->feuserSessionService->set($filter, 'filter');
        $this->feuserSessionService->set($_POST['tx_a2gproducts_a2gproductlist']['searchField'], 'searchField');

        // duplicate from listAction make a new function !!
        if (!empty($filter) || $_POST['tx_a2gproducts_a2gproductlist']['searchField'] !== '') {
            if ((int) $settings['ajaxListLimit'] === 0) {
                $products = $this->productsRepository->filter($catUids, $filter, $_POST['tx_a2gproducts_a2gproductlist']['searchField'], 0, 0, $filters);
            } else {
                $products = $this->productsRepository->filter($catUids, $filter, $_POST['tx_a2gproducts_a2gproductlist']['searchField'], $settings['ajaxListLimit'] + 2, (int) $_POST['tx_a2gproducts_a2gproductlist']['loads'] * $settings['ajaxListLimit'], $filters);
            }
        } else {
            if ((int) $settings['ajaxListLimit'] === 0) {
                $products = $this->productsRepository->getFromCategories($catUids);
            } else {
                $products = $this->productsRepository->getFromCategories($catUids, $settings['ajaxListLimit'] + 2, (int) $_POST['tx_a2gproducts_a2gproductlist']['loads'] * $settings['ajaxListLimit']);
            }
        }

        $this->view->assign('products', $products);
        $this->view->assign('filters', $filters);
        $this->view->assign('settings', $settings);
    }

    static protected function isAllowedCategory(\A2G\A2gProducts\Domain\Model\Products $product, $categories){
        
    }

    /**
     * action show
     *
     * @param \A2G\A2gProducts\Domain\Model\Products $product
     * @param \A2G\A2gProducts\Domain\Model\ProductVariants $productVariant
     * @return string|object|null|void
     */
    public function showAction(\A2G\A2gProducts\Domain\Model\Products $product = null, \A2G\A2gProducts\Domain\Model\ProductVariants $productVariant = null) {



        if($product === null){
            return GeneralUtility::makeInstance(\TYPO3\CMS\Frontend\Controller\ErrorController::class)
    ->pageNotFoundAction(
        $GLOBALS['TYPO3_REQUEST'],
        'Shop Product not found!',
        ['code' => \TYPO3\CMS\Frontend\Page\PageAccessFailureReasons::INVALID_PAGE_ARGUMENTS]
    );
        }

        $titleProvider = GeneralUtility::makeInstance(A2gPageTitleProvider::class);
        $contentObject = GeneralUtility::makeInstance(ContentObjectRenderer::class);
        $description = $contentObject->crop($product->getDescription(), 125 . '|...|' . true);
        $imageUri=null;
        if ($product->getImage() !== null) {
            $processedImage = $this->imageService->applyProcessingInstructions(
                    $this->imageService->getImage('', $product->getImage(), false),
                    ['width' => '1200c', 'height' => '630']
            );
            $imageUri = $this->imageService->getImageUri($processedImage, true);
        }

        $this->view->assign('product', $product);

        if($productVariant === null){
            $uri = $this->uriBuilder->reset()->setCreateAbsoluteUri(true)->setTargetPageUid(
                        $GLOBALS['TSFE']->id)->uriFor('show', ['product' => $product], 'Products', 'a2gShop', 'A2gproductsdetail');
        } else {
            $uri = $this->uriBuilder->reset()->setCreateAbsoluteUri(true)->setTargetPageUid(
                    $GLOBALS['TSFE']->id)->uriFor('show', ['product' => $product, 'productVariant' => $productVariant], 'Products', 'a2gShop', 'A2gproductsdetail');
        }
        if($productVariant !== null &&  array_key_exists('pageTitleSuffix', $this->settings) && $this->settings['pageTitleSuffix'] !== '' && (strlen($product->getTitle()) + (strlen($productVariant->getTitle()) + strlen($this->settings['pageTitleSuffix'])) < 65 )){
            $titleProvider->setTitle($product->getTitle() . ' ' .$productVariant->getTitle() .' - ' . $this->settings['pageTitleSuffix']);
            $this->setMetaTags($product->getTitle(). ' ' .$productVariant->getTitle()  .' - ' . $this->settings['pageTitleSuffix'], $description, $uri, $imageUri);
        } else  if($productVariant !== null && (strlen($product->getTitle()) + strlen($productVariant->getTitle())) < 65 ){
            $titleProvider->setTitle($product->getTitle() . ' ' .$productVariant->getTitle());
            $this->setMetaTags($product->getTitle(). ' ' .$productVariant->getTitle(), $description, $uri, $imageUri);
        }else if(array_key_exists('pageTitleSuffix', $this->settings) && $this->settings['pageTitleSuffix'] !== '' && (strlen($product->getTitle()) + strlen($this->settings['pageTitleSuffix'])) < 66 ){
            $titleProvider->setTitle($product->getTitle() .' - ' . $this->settings['pageTitleSuffix']);
            $this->setMetaTags($product->getTitle() .' - ' . $this->settings['pageTitleSuffix'], $description, $uri, $imageUri);
        } else {
            $titleProvider->setTitle($product->getTitle());
            $this->setMetaTags($product->getTitle(), $description, $uri, $imageUri);
        }
        if(!array_key_exists('listPage', $this->settings)){
            $this->view->assign('contentObjectUid', $this->configurationManager->getContentObject()->data['uid']);
        }
    }

}
