<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Domain\Model;


/**
 * This file is part of the "altogether Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * Categories
 */
class Categories extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

    /**
     * title
     *
     * @var string
     */
    protected $title = '';

    /**
     * description
     *
     * @var string
     */
    protected $description = '';

    /**
     * image
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $image = null;

    /**
     * relFilters
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\Filters>
     */
    protected $relFilters = null;

    /**
     * __construct
     */
    public function __construct()
    {

        // Do not remove the next line: It would break the functionality
        $this->initializeObject();
    }

    /**
     * Initializes all ObjectStorage properties when model is reconstructed from DB (where __construct is not called)
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    public function initializeObject()
    {
        $this->relFilters = $this->relFilters ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Returns the title
     *
     * @return string $title
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * Sets the title
     *
     * @param string $title
     * @return void
     */
    public function setTitle(string $title)
    {
        $this->title = $title;
    }

    /**
     * Returns the description
     *
     * @return string $description
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Sets the description
     *
     * @param string $description
     * @return void
     */
    public function setDescription(string $description)
    {
        $this->description = $description;
    }

    /**
     * Returns the image
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $image
     */
    public function getImage()
    {
        return $this->image;
    }

    /**
     * Sets the image
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $image
     * @return void
     */
    public function setImage(\TYPO3\CMS\Extbase\Domain\Model\FileReference $image)
    {
        $this->image = $image;
    }

    /**
     * Adds a Filters
     *
     * @param \A2G\A2gProducts\Domain\Model\Filters $relFilter
     * @return void
     */
    public function addRelFilter(\A2G\A2gProducts\Domain\Model\Filters $relFilter)
    {
        $this->relFilters->attach($relFilter);
    }

    /**
     * Removes a Filters
     *
     * @param \A2G\A2gProducts\Domain\Model\Filters $relFilterToRemove The Filters to be removed
     * @return void
     */
    public function removeRelFilter(\A2G\A2gProducts\Domain\Model\Filters $relFilterToRemove)
    {
        $this->relFilters->detach($relFilterToRemove);
    }

    /**
     * Returns the relFilters
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\Filters> $relFilters
     */
    public function getRelFilters()
    {
        return $this->relFilters;
    }

    /**
     * Sets the relFilters
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\Filters> $relFilters
     * @return void
     */
    public function setRelFilters(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relFilters)
    {
        $this->relFilters = $relFilters;
    }
}
