<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Domain\Model;


/**
 * This file is part of the "altogether Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * Filters
 */
class Filters extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

    /**
     * dbOperations
     * 
     * @var array
     */
    static public $dbOperations = [
        1 => '=',
        2 => '>',
        3 => '<',
        4 => '>=',
        5 => '<=',
        6 => 'IN'
    ];
    

    /**
     * dbType
     * 
     * @var array
     */
    static public $dbType = [
        1 => 'range ( uses min and max value )',
        2 => 'compare ( uses just min value )',
        3 => 'useFilterOptions'
    ];
    
    /**
     * title
     *
     * @var string
     */
    protected $title = '';

    /**
     * filterType
     *
     * @var int
     */
    protected $filterType = '';

    /**
     * dbOperationMax
     *
     * @var int
     */
    protected $dbOperationMax = 0;

    /**
     * dbOperationMin
     *
     * @var int
     */
    protected $dbOperationMin = '';

    /**
     * relFilterOption
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\FilterOptions>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     */
    protected $relFilterOption = null;

    /**
     * Returns the title
     *
     * @return string $title
     */
    public function getTitle()
    {
        return $this->title;
    }

    /**
     * Sets the title
     *
     * @param string $title
     * @return void
     */
    public function setTitle(string $title)
    {
        $this->title = $title;
    }

    /**
     * Returns the filterType
     *
     * @return string $filterType
     */
    public function getFilterType()
    {
        return $this->filterType;
    }

    /**
     * Sets the filterType
     *
     * @param string $filterType
     * @return void
     */
    public function setFilterType(string $filterType)
    {
        $this->filterType = $filterType;
    }

    /**
     * Returns the dbOperationMax
     *
     * @return int $dbOperationMax
     */
    public function getDbOperationMax()
    {
        return $this->dbOperationMax;
    }

    /**
     * Sets the dbOperationMax
     *
     * @param int $dbOperationMax
     * @return void
     */
    public function setDbOperationMax(int $dbOperationMax)
    {
        $this->dbOperationMax = $dbOperationMax;
    }

    /**
     * Returns the dbOperationMin
     *
     * @return int dbOperationMin
     */
    public function getDbOperationMin()
    {
        return $this->dbOperationMin;
    }

    /**
     * Sets the dbOperationMin
     *
     * @param string $dbOperationMin
     * @return void
     */
    public function setDbOperationMin(string $dbOperationMin)
    {
        $this->dbOperationMin = $dbOperationMin;
    }

    /**
     * __construct
     */
    public function __construct()
    {

        // Do not remove the next line: It would break the functionality
        $this->initializeObject();
    }

    /**
     * Initializes all ObjectStorage properties when model is reconstructed from DB (where __construct is not called)
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    public function initializeObject()
    {
        $this->relFilterOption = $this->relFilterOption ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Adds a FilterOptions
     *
     * @param \A2G\A2gProducts\Domain\Model\FilterOptions $relFilterOption
     * @return void
     */
    public function addRelFilterOption(\A2G\A2gProducts\Domain\Model\FilterOptions $relFilterOption)
    {
        $this->relFilterOption->attach($relFilterOption);
    }

    /**
     * Removes a FilterOptions
     *
     * @param \A2G\A2gProducts\Domain\Model\FilterOptions $relFilterOptionToRemove The FilterOptions to be removed
     * @return void
     */
    public function removeRelFilterOption(\A2G\A2gProducts\Domain\Model\FilterOptions $relFilterOptionToRemove)
    {
        $this->relFilterOption->detach($relFilterOptionToRemove);
    }

    /**
     * Returns the relFilterOption
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\FilterOptions> $relFilterOption
     */
    public function getRelFilterOption()
    {
        return $this->relFilterOption;
    }

    /**
     * Sets the relFilterOption
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\FilterOptions> $relFilterOption
     * @return void
     */
    public function setRelFilterOption(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relFilterOption)
    {
        $this->relFilterOption = $relFilterOption;
    }
}
