<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Domain\Model;

use A2G\A2gProducts\Domain\Model\{
    Products,
    MasonryItem
};
use TYPO3\CMS\Extbase\Persistence\Generic\LazyLoadingProxy;

/**
 * This file is part of the "Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * Ingredients
 */
class Ingredients extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

    /**
     * title
     *
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $title = '';

    /**
     * pathSegment
     * 
     * @var string
     */
    protected $pathSegment = '';
    
    /**
     * description
     *
     * @var string
     */
    protected $description = '';

    /**
     * image
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $image = null;

    /**
     * images
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\FileReference>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $images = null;

    /**
     * gallery
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<MasonryItem>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $gallery = null;


    /**
     * relProducts
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Products>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $relProducts = null;


    /**
     * Returns the title
     *
     * @return string $title
     */
    public function getTitle() {
        return $this->title;
    }

    /**
     * Sets the title
     *
     * @param string $title
     * @return void
     */
    public function setTitle(string $title) {
        $this->title = $title;
    }

    /**
     * Returns the pathSegment
     * 
     * @return string $pathSegment
     */
    public function getPathSegment()
    {
        return $this->pathSegment;
    }

    /**
     * Sets the pathSegment
     * 
     * @param string $pathSegment
     * @return void
     */
    public function setPathSegment($pathSegment)
    {
        $this->pathSegment = $pathSegment;
    }
    
    /**
     * Returns the description
     *
     * @return string $description
     */
    public function getDescription() {
        return $this->description;
    }

    /**
     * Sets the description
     *
     * @param string $description
     * @return void
     */
    public function setDescription(string $description) {
        $this->description = $description;
    }

    /**
     * Returns the image
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $image
     */
    public function getImage() {
        if ($this->image instanceof LazyLoadingProxy) {
            $this->image = $this->image->_loadRealInstance();
        }
        return $this->image;
    }

    /**
     * Sets the image
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $image
     * @return void
     */
    public function setImage(\TYPO3\CMS\Extbase\Domain\Model\FileReference $image) {
        $this->image = $image;
    }

    /**
     * Returns the images
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $images
     */
    public function getImages() {
        return $this->images;
    }

    /**
     * Sets the images
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $images
     * @return void
     */
    public function setImages(\TYPO3\CMS\Extbase\Domain\Model\FileReference $images) {
        $this->images = $images;
    }

    /**
     * Returns the gallery
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $gallery
     */
    public function getGallery() {
        return $this->gallery;
    }

    /**
     * Sets the gallery
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $gallery
     * @return void
     */
    public function setGallery(\TYPO3\CMS\Extbase\Domain\Model\FileReference $gallery) {
        $this->gallery = $gallery;
    }

    
    /**
     * __construct
     */
    public function __construct() {

        // Do not remove the next line: It would break the functionality
        $this->initializeObject();
    }

    /**
     * Initializes all ObjectStorage properties when model is reconstructed from DB (where __construct is not called)
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    public function initializeObject() {
        $this->gallery = $this->gallery ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->relProducts = $this->relProducts ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }
    
    
    /**
     * Adds a MasonryItem
     *
     * @param MasonryItem $relMasonryItem
     * @return void
     */
    public function addGallerys(MasonryItem $relMasonryItem) {
        $this->gallery->attach($relMasonryItem);
    }

    /**
     * Removes a MasonryItem
     *
     * @param MasonryItem $relMasonryItem The MasonryItem to be removed
     * @return void
     */
    public function removeGallerys(MasonryItem $relMasonryItem) {
        $this->gallery->detach($relMasonryItem);
    }

    /**
     * Returns the gallery
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<MasonryItem> $gallery
     */
    public function getGallerys() {
        return $this->gallery;
    }

    /**
     * Sets the gallery
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<MasonryItem> $gallery
     * @return void
     */
    public function setGallerys(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $gallery) {
        $this->gallery = $gallery;
    }
    
    /**
     * Adds a Products
     *
     * @param Products $relProducts
     * @return void
     */
    public function addRelProducts(Products $relProducts) {
        $this->relProducts->attach($relProducts);
    }

    /**
     * Removes a Products
     *
     * @param Products $relProducts The Products to be removed
     * @return void
     */
    public function removeRelProducts(Products $relProducts) {
        $this->relProducts->detach($relProducts);
    }

    /**
     * Returns the relProducts
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Products> $relProducts
     */
    public function getRelProducts() {
        return $this->relProducts;
    }

    /**
     * Sets the relProducts
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Products> $relProducts
     * @return void
     */
    public function setRelProducts(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relProducts) {
        $this->relProducts = $relProducts;
    }
}
