<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Domain\Model;

use A2G\A2gProducts\Domain\Model\{
    Ingredients,
    ProductVariants,
    MasonryItem,
    Brands
};
use TYPO3\CMS\Extbase\Persistence\Generic\LazyLoadingProxy;

/**
 * This file is part of the "Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * Products
 */
class Products extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

    /**
     * nr
     *
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $nr = '';

    /**
     * title
     *
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $title = '';

    /**
     * pathSegment
     * 
     * @var string
     */
    protected $pathSegment = '';

    /**
     * description
     *
     * @var string
     */
    protected $description = '';

    /**
     * descriptionHtml
     *
     * @var string
     */
    protected $descriptionHtml = '';

    /**
     * image
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $image = null;

    /**
     * images
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\FileReference>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $images = null;

    /**
     * gallery
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<MasonryItem>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $gallery = null;


    /**
     * seoImage1x1
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $seoImage1x1 = null;
    /**
     * seoImage4x3
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $seoImage4x3 = null;
    /**
     * seoImage16x9
     *
     * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $seoImage16x9 = null;
    
    /**
     * relCategory
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\Categories>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $relCategory = null;

    /**
     * relFilterValues
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\FilterValues>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $relFilterValues = null;

    /**
     * relIngredients
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Ingredients>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $relIngredients = null;

    /**
     * relProductvariants
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<ProductVariants>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $relProductvariants = null;

    /**
     * relFilterValues
     *
     * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\SelectableAttribute>
     * @TYPO3\CMS\Extbase\Annotation\ORM\Cascade("remove")
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $relSelectableattribute = null;
    
    /**
     * relBrand
     *
     * @var Brands
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $relBrand = null;

    /**
     * Returns the nr
     *
     * @return string $nr
     */
    public function getNr() {
        return $this->nr;
    }

    /**
     * Sets the nr
     *
     * @param string $nr
     * @return void
     */
    public function setNr(string $nr) {
        $this->nr = $nr;
    }

    /**
     * Returns the title
     *
     * @return string $title
     */
    public function getTitle() {
        return $this->title;
    }

    /**
     * Sets the title
     *
     * @param string $title
     * @return void
     */
    public function setTitle(string $title) {
        $this->title = $title;
    }

    /**
     * Returns the pathSegment
     * 
     * @return string $pathSegment
     */
    public function getPathSegment() {
        return $this->pathSegment;
    }

    /**
     * Sets the pathSegment
     * 
     * @param string $pathSegment
     * @return void
     */
    public function setPathSegment($pathSegment) {
        $this->pathSegment = $pathSegment;
    }

    /**
     * Returns the description
     *
     * @return string $description
     */
    public function getDescription() {
        return $this->description;
    }

    /**
     * Sets the description
     *
     * @param string $description
     * @return void
     */
    public function setDescription(string $description) {
        $this->description = $description;
    }

    public function getDescriptionHtml(): string {
        return $this->descriptionHtml;
    }

    public function setDescriptionHtml(string $descriptionHtml): void {
        $this->descriptionHtml = $descriptionHtml;
    }

    /**
     * Returns the image
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $image
     */
    public function getImage() {
        if ($this->image instanceof LazyLoadingProxy) {
            $this->image = $this->image->_loadRealInstance();
        }
        return $this->image;
    }

    /**
     * Sets the image
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $image
     * @return void
     */
    public function setImage(\TYPO3\CMS\Extbase\Domain\Model\FileReference $image) {
        $this->image = $image;
    }

    /**
     * Returns the images
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $images
     */
    public function getImages() {
        return $this->images;
    }

    /**
     * Sets the images
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $images
     * @return void
     */
    public function setImages(\TYPO3\CMS\Extbase\Domain\Model\FileReference $images) {
        $this->images = $images;
    }

    /**
     * Returns the gallery
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $gallery
     */
    public function getGallery() {
        return $this->gallery;
    }

    /**
     * Sets the gallery
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $gallery
     * @return void
     */
    public function setGallery(\TYPO3\CMS\Extbase\Domain\Model\FileReference $gallery) {
        $this->gallery = $gallery;
    }

    
    
    /**
     * Returns the seoImage1x1
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage1x1
     */
    public function getSeoImage1x1() {
        if ($this->seoImage1x1 instanceof LazyLoadingProxy) {
            $this->seoImage1x1 = $this->seoImage1x1->_loadRealInstance();
        }
        return $this->seoImage1x1;
    }

    /**
     * Sets the seoImage1x1
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage1x1
     * @return void
     */
    public function setSeoImage1x1(\TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage1x1) {
        $this->seoImage1x1 = $seoImage1x1;
    }

    /**
     * Returns the seoImage4x3
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage4x3
     */
    public function getSeoImage4x3() {
        if ($this->seoImage4x3 instanceof LazyLoadingProxy) {
            $this->seoImage4x3 = $this->seoImage4x3->_loadRealInstance();
        }
        return $this->seoImage4x3;
    }

    /**
     * Sets the seoImage4x3
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage4x3
     * @return void
     */
    public function setSeoImage4x3(\TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage4x3) {
        $this->seoImage4x3 = $seoImage4x3;
    }

    /**
     * Returns the seoImage16x9
     *
     * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage16x9
     */
    public function getSeoImage16x9() {
        if ($this->seoImage16x9 instanceof LazyLoadingProxy) {
            $this->seoImage16x9 = $this->seoImage16x9->_loadRealInstance();
        }
        return $this->seoImage16x9;
    }

    /**
     * Sets the seoImage16x9
     *
     * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage16x9
     * @return void
     */
    public function setSeoImage16x9(\TYPO3\CMS\Extbase\Domain\Model\FileReference $seoImage16x9) {
        $this->seoImage16x9 = $seoImage16x9;
    }

    
    
    
    /**
     * __construct
     */
    public function __construct() {

        // Do not remove the next line: It would break the functionality
        $this->initializeObject();
    }

    /**
     * Initializes all ObjectStorage properties when model is reconstructed from DB (where __construct is not called)
     * Do not modify this method!
     * It will be rewritten on each save in the extension builder
     * You may modify the constructor of this class instead
     *
     * @return void
     */
    public function initializeObject() {
        $this->relCategory = $this->relCategory ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->relFilterValues = $this->relFilterValues ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->relIngredients = $this->relIngredients ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->relSelectableattribute = $this->relSelectableattribute ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->relProductvariants = $this->relProductvariants ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
        $this->gallery = $this->gallery ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
//        $this->relBrand = $this->relBrand ?: new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
    }

    /**
     * Adds a Categories
     *
     * @param \A2G\A2gProducts\Domain\Model\Categories $relCategory
     * @return void
     */
    public function addRelCategory(\A2G\A2gProducts\Domain\Model\Categories $relCategory) {
        $this->relCategory->attach($relCategory);
    }

    /**
     * Removes a Categories
     *
     * @param \A2G\A2gProducts\Domain\Model\Categories $relCategoryToRemove The Categories to be removed
     * @return void
     */
    public function removeRelCategory(\A2G\A2gProducts\Domain\Model\Categories $relCategoryToRemove) {
        $this->relCategory->detach($relCategoryToRemove);
    }

    /**
     * Returns the relCategory
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\Categories> $relCategory
     */
    public function getRelCategory() {
        return $this->relCategory;
    }

    /**
     * Sets the relCategory
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\Categories> $relCategory
     * @return void
     */
    public function setRelCategory(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relCategory) {
        $this->relCategory = $relCategory;
    }

    /**
     * Adds a FilterValues
     *
     * @param \A2G\A2gProducts\Domain\Model\FilterValues $relFilterValue
     * @return void
     */
    public function addRelFilterValue(\A2G\A2gProducts\Domain\Model\FilterValues $relFilterValue) {
        $this->relFilterValues->attach($relFilterValue);
    }

    /**
     * Removes a FilterValues
     *
     * @param \A2G\A2gProducts\Domain\Model\FilterValues $relFilterValueToRemove The FilterValues to be removed
     * @return void
     */
    public function removeRelFilterValue(\A2G\A2gProducts\Domain\Model\FilterValues $relFilterValueToRemove) {
        $this->relFilterValues->detach($relFilterValueToRemove);
    }

    /**
     * Returns the relFilterValues
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\FilterValues> $relFilterValues
     */
    public function getRelFilterValues() {
        return $this->relFilterValues;
    }

    /**
     * Sets the relFilterValues
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\FilterValues> $relFilterValues
     * @return void
     */
    public function setRelFilterValues(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relFilterValues) {
        $this->relFilterValues = $relFilterValues;
    }

    /**
     * Adds a SelectableAttribute
     *
     * @param \A2G\A2gProducts\Domain\Model\SelectableAttribute $relSelectableattribute
     * @return void
     */
    public function addRelSelectableattribute(\A2G\A2gProducts\Domain\Model\SelectableAttribute $relSelectableattribute) {
        $this->relSelectableattribute->attach($relSelectableattribute);
    }

    /**
     * Removes a SelectableAttribute
     *
     * @param \A2G\A2gProducts\Domain\Model\SelectableAttribute $relSelectableattributeToRemove The SelectableAttribute to be removed
     * @return void
     */
    public function removeRelSelectableattribute(\A2G\A2gProducts\Domain\Model\SelectableAttribute $relSelectableattributeToRemove) {
        $this->relSelectableattribute->detach($relSelectableattributeToRemove);
    }

    /**
     * Returns the relSelectableAttribute
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\SelectableAttribute> $relSelectableattribute
     */
    public function getRelSelectableattribute() {
        return $this->relSelectableattribute;
    }

    /**
     * Sets the relSelectableAttribute
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\A2G\A2gProducts\Domain\Model\SelectableAttribute> $relSelectableattribute
     * @return void
     */
    public function setRelSelectableattribute(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relSelectableattribute) {
        $this->relSelectableattribute = $relSelectableattribute;
    }

    /**
     * Adds a FilterValues
     *
     * @param Ingredients $relIngredients
     * @return void
     */
    public function addRelIngredients(Ingredients $relIngredients) {
        $this->relIngredients->attach($relIngredients);
    }

    /**
     * Removes a FilterValues
     *
     * @param Ingredients $relIngredients The Ingredients to be removed
     * @return void
     */
    public function removeRelIngredients(Ingredients $relIngredients) {
        $this->relIngredients->detach($relIngredients);
    }

    /**
     * Returns the relIngredients
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Ingredients> $relIngredients
     */
    public function getRelIngredients() {
        return $this->relIngredients;
    }

    /**
     * Sets the relIngredients
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<Ingredients> $relIngredients
     * @return void
     */
    public function setRelIngredients(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relIngredients) {
        $this->relIngredients = $relIngredients;
    }

    /**
     * Adds a MasonryItem
     *
     * @param MasonryItem $relMasonryItem
     * @return void
     */
    public function addGallerys(MasonryItem $relMasonryItem) {
        $this->gallery->attach($relMasonryItem);
    }

    /**
     * Removes a MasonryItem
     *
     * @param MasonryItem $relMasonryItem The MasonryItem to be removed
     * @return void
     */
    public function removeGallerys(MasonryItem $relMasonryItem) {
        $this->gallery->detach($relMasonryItem);
    }

    /**
     * Returns the gallery
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<MasonryItem> $gallery
     */
    public function getGallerys() {
        return $this->gallery;
    }

    /**
     * Sets the gallery
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<MasonryItem> $gallery
     * @return void
     */
    public function setGallerys(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $gallery) {
        $this->gallery = $gallery;
    }

//    
//    public function getRelProductvariants() {
//        return $this->relProductvariants;
//    }
//
//    public function setRelProductvariants($relProductvariants): void {
//        $this->relProductvariants = $relProductvariants;
//    }
//
//    

    /**
     * Adds a FilterValues
     *
     * @param ProductVariants $relProductvariants
     * @return void
     */
    public function addRelProductvariants($relProductvariants) {
        $this->relProductvariants->attach($relProductvariants);
    }

    /**
     * Removes a FilterValues
     *
     * @param ProductVariants $relProductvariants The ProductVariants to be removed
     * @return void
     */
    public function removeRelProductvariants($relProductvariants) {
        $this->relProductvariants->detach($relProductvariants);
    }

    /**
     * Returns the relProductvariants
     *
     * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<ProductVariants> $relProductvariants
     */
    public function getRelProductvariants() {
        return $this->relProductvariants;
    }

    /**
     * Sets the relProductvariants
     *
     * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<ProductVariants> $relProductvariants
     * @return void
     */
    public function setRelProductvariants(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $relProductvariants) {
        $this->relProductvariants = $relProductvariants;
    }
    
    /**
     * Returns the relBrand
     * 
     * @return Brands|null
     */
    public function getRelBrand():?Brands {
        if ($this->relBrand instanceof LazyLoadingProxy) {
            $this->relBrand = $this->relBrand->_loadRealInstance();
        }
        return $this->relBrand;
    }

    /**
     * Sets the relBrand
     * 
     * @param Brands|null $relBrand
     * @return void
     */
    public function setRelBrand(?Brands $relBrand) {
        $this->relBrand = $relBrand;
    }
}
