<?php

declare(strict_types=1);

namespace A2G\A2gProducts\Domain\Repository;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Context\Context;

/**
 * This file is part of the "Products" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * (c) 2021 Raphael Martin <raphy.martin@gmail.com>, none
 */

/**
 * The repository for Products
 */
class FiltersRepository extends \TYPO3\CMS\Extbase\Persistence\Repository {
    /**
 * @var array
 */

    protected $defaultOrderings = [
        'sorting' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
    ];

    /**
     * @param array $categoryUids
     */
    public function getFromCategories(array $categoryUids = []) {
        $languageAspect = GeneralUtility::makeInstance(Context::class)->getAspect('language');
        $sys_language_uid = $languageAspect->getId();

        $query = $this->createQuery();
        $sql = 'SELECT DISTINCT fT.*  FROM tx_a2gproducts_domain_model_filters AS fT '
                . 'JOIN tx_a2gproducts_categories_filters_mm AS mmT ON mmT.uid_foreign  = fT.uid' 
                . ' WHERE fT.hidden=0 AND fT.deleted=0 '
                . (empty($categoryUids) ? '' :'AND mmT.uid_local IN (' . implode(',', $categoryUids ) . ') ')
                . 'AND fT.sys_language_uid IN (:dcSysLanguageUid, -1) ORDER BY fT.sorting';
        $arguments = [
            ':dcSysLanguageUid' => $sys_language_uid
        ];
        $query->statement($sql, $arguments);
        return $query->execute();
    }

}
