/* 
 * Copyright (C) 2021 Raphael Martin <r.martin@felder-group.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * GaProducts
 */
var GaProducts = function () {
    'use strict';
    var Constructor = function (itemListId, itemListName, GaProductObject) {
        this.item_list_id = itemListId;
        this.item_list_name = itemListName;
        this.arr = [];
        if (typeof (GaProductObject) === 'object' && GaProductObject instanceof GaProduct) {
            this.arr.push(GaProductObject);
        }
    };
    Constructor.prototype.push = function (GaProductObject) {
        if (typeof (GaProductObject) === 'object' && GaProductObject instanceof GaProduct) {
            this.arr.push(GaProductObject);
        } else {
            console.log('GaProductObject must be a instanceof GaProduct');
        }
    };
    Constructor.prototype.viewItemList = function () {
        gtag("event", "view_item_list", {
            item_list_id: this.item_list_id,
            item_list_name: this.item_list_name,
            items: JSON.stringify(this.arr)
        });
    };
    Constructor.prototype.impressions = function (currencyCode = 'EUR', list = 'Search Results') {
        if (typeof ga === 'function' && typeof dataLayer === 'object' && this.arr.length > 0) {
            var products = [];
            for (var i = 0; i < this.arr.length; i++) {
                products.push({
                    'name': this.arr[i].name, // Name or ID is required.
                    'id': this.arr[i].id,
                    'price': this.arr[i].price,
                    'brand': this.arr[i].brand,
                    'category': this.arr[i].category,
                    'variant': this.arr[i].variant,
                    'list': list,
                    'position': this.arr[i].position
                });
            }
            dataLayer.push({
                'ecommerce': {
                    'currencyCode': currencyCode,
                    'impressions': products
                }
            });
        } else {
            console.log('GaProducts.productClick requiers ga and dataLayer');
    }
    };
    Constructor.prototype.addToCart = function (currencyCode = 'EUR') {
        if (typeof ga === 'function' && typeof dataLayer === 'object' && this.arr.length > 0) {
            var products = [];
            for (var i = 0; i < this.arr.length; i++) {
                products.push({
                    'name': this.arr[i].name, // Name or ID is required.
                    'id': this.arr[i].id,
                    'price': this.arr[i].price,
                    'brand': this.arr[i].brand,
                    'category': this.arr[i].category,
                    'variant': this.arr[i].variant,
                    'quantity': this.arr[i].quantity
                });
            }
            dataLayer.push({
                'event': 'addToCart',
                'ecommerce': {
                    'currencyCode': currencyCode.toUpperCase(),
                    'add': {
                        'products': products
                    }
                }
            });
        } else {
            console.log('GaProducts.addToCart requiers ga and dataLayer');
    }
    };
    Constructor.prototype.removeFromCart = function () {
        if (typeof ga === 'function' && typeof dataLayer === 'object' && this.arr.length > 0) {
            var products = [];
            for (var i = 0; i < this.arr.length; i++) {
                products.push({
                    'name': this.arr[i].name, // Name or ID is required.
                    'id': this.arr[i].id,
                    'price': this.arr[i].price,
                    'brand': this.arr[i].brand,
                    'category': this.arr[i].category,
                    'variant': this.arr[i].variant,
                    'quantity': this.arr[i].quantity
                });
            }
            dataLayer.push({
                'event': 'removeFromCart',
                'ecommerce': {
                    'remove': {
                        'products': products
                    }
                }
            });
        } else {
            console.log('GaProducts.removeFromCart requiers ga and dataLayer');
        }
    };
    return Constructor;
}(GaProduct);
